package ansmart;

import gate.Factory;
import gate.FeatureMap;
import gate.Gate;
import gate.ProcessingResource;
import gate.GateExtractor.EnumProject;
import gate.creole.POSTagger;
import gate.creole.ResourceInstantiationException;
import gate.creole.SerialAnalyserController;
import gate.creole.gazetteer.DefaultGazetteer;
import gate.creole.gazetteer.Gazetteer;
import gate.creole.morph.Morph;
import gate.creole.ontology.Ontology;
import gate.creole.tokeniser.DefaultTokeniser;
import gate.util.CommonMethod;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

//import uk.org.ccgv.doc4u.exception.GenericException;

public class PipelineBuilder {
	private static final String SERIAL_CONTROLLER = "gate.creole.SerialAnalyserController";
	private static final String FLEXIBLE_GAZETTEER = "gate.creole.gazetteer.FlexibleGazetteer";
	private static final String ANNIE_GAZETTEER = "gate.creole.gazetteer.DefaultGazetteer";
	private static final String ONTO_ROOT_GAZETTEER = "gate.clone.ql.OntoRootGaz";
	// private static final String ONTOLOGY_LR =
	// "gate.creole.ontology.owlim.OWLIMOntologyLR";
	private static final String ONTOLOGY_OWLIM = "gate.creole.ontology.impl.sesame.OWLIMOntology";
	private static final String JAPE_TRANSDUCER = "gate.creole.Transducer"; // alternate com.ontotext.gate.japec.JapecTransducer
	
	private static final String HASH_GAZETTEER = "com.ontotext.gate.gazetteer.HashGazetteer";//for domain dic
	
	private static final String DOCUMENT_RESET = "gate.creole.annotdelete.AnnotationDeletePR";
	private static final String REGEX_SENTENCE_SPLITTER = "gate.creole.splitter.RegexSentenceSplitter";
	private static final String ENGLISH_TOKENISER = "gate.creole.tokeniser.DefaultTokeniser";
	private static final String POS = "gate.creole.POSTagger";
	private static final String MORPHER = "gate.creole.morph.Morph";

	
	private ProcessingResource tokenizerPR ;
	private ProcessingResource posPR;
	private ProcessingResource morpherPR;
	//private URL ontoUrl;
	private Ontology ontology;
	
	private int mode;//0 stand alone 1 web
	private static final String[] PR_CLASSES = {
		"gate.creole.annotdelete.AnnotationDeletePR"      // 0 - Document Reset PR
		, "gate.creole.splitter.RegexSentenceSplitter"    // 1 - RegEx Sentence Splitter
		, "gate.creole.tokeniser.DefaultTokeniser"        // 2 - ANNIE English Tokeniser
		, "gate.creole.POSTagger"                         // 3 - ANNIE POS Tagger
		, "gate.creole.morph.Morph"                       // 4 - GATE Morphological analyser
		, FLEXIBLE_GAZETTEER                              // Flexible Gazetteer
		// , ANNIE_GAZETEER                               // ANNIE Gazetteer
		, "gate.creole.ANNIETransducer"                   // ANNIE NE Transducer
		, HASH_GAZETTEER                                   //Hash Gazetteer
		, JAPE_TRANSDUCER                                 // JAPE Transducer
		
	};
	
	public PipelineBuilder(int mode){
		this.mode = mode;
		//this.mode = 0;
	}
	private ProcessingResource getDocumentResetPR() throws ResourceInstantiationException{
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(DOCUMENT_RESET, params);
		return rtn;
	}
	
	private ProcessingResource getTokenizerPR() throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(ENGLISH_TOKENISER, params);
		return rtn;
	}
	private ProcessingResource getSentenceSplitterPR() throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(REGEX_SENTENCE_SPLITTER, params);
		return rtn;
	}
	private ProcessingResource getPosPR()throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(POS, params);
		return rtn;
	}
	private ProcessingResource getMorphPR()throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(MORPHER, params);
		return rtn;
	}
	
	/*
	private Vector<ProcessingResource> getFlexibleGazetteerList()throws ResourceInstantiationException{
		Vector<ProcessingResource> rtn = null;
		String ontoFiles = CommonMethod.getOntoFileNames();
		String tmpFile,tmpFilePath;
		ProcessingResource tmpPR;
		if(ontoFiles !=null && !ontoFiles.equals("")){
			rtn = new Vector<ProcessingResource>();
			 StringTokenizer st = new StringTokenizer(ontoFiles,",");
			 while(st.hasMoreTokens()){//for every onto file
				 tmpFile = st.nextToken();
				 tmpPR = getFlexibleGazetteer(tmpFile);
				 rtn.add(tmpPR);
			 }
		}
		return rtn;
	}*/
    //ontology file name
	private  ProcessingResource getFlexibleGazetteer(String filePath)throws ResourceInstantiationException
	{	
		ProcessingResource rtn = null;
		//-----------create ontology param
		FeatureMap paramsOnto,paramsRGPR,paramsFlexPR;
		paramsOnto = Factory.newFeatureMap(); // use default parameters
		//String filePath = getOntologyFilePathWrite(fileName);
		//-----for test
		//this.mode =1;
		URL fUrl = null;
		try {
			if(this.mode ==0){
				fUrl = new URL("file:/" + filePath);
			}else{
				fUrl = new URL( filePath);
			}
				
		} catch (MalformedURLException e) {
			System.out.println(e);
			e.printStackTrace();
				//throw new GenericException(e.getMessage(), "Back", "dashboard");
		}
		
		paramsOnto.put("rdfXmlURL", fUrl);
		
		Ontology ontology = (Ontology) Factory.createResource(ONTOLOGY_OWLIM,paramsOnto);
		this.ontology = ontology;//ontology.getURL();
		getTokenizerPR();
		if(tokenizerPR==null){
			tokenizerPR = this.getTokenizerPR();
		}
		if(posPR == null){
			posPR = this.getPosPR();
		}
		if(morpherPR == null){
			morpherPR = this.getMorphPR();
		}
		//------------create pr param----------------
		paramsRGPR = Factory.newFeatureMap(); // use default parameters
		paramsRGPR.put("ontology", ontology); // gate.creole.ontology.Ontology
		paramsRGPR.put("tokeniser", (DefaultTokeniser) this.tokenizerPR); // gate.creole.tokeniser.DefaultTokeniser
		paramsRGPR.put("posTagger", (POSTagger) this.posPR); // gate.creole.POSTagger
		paramsRGPR.put("morpher", (Morph) this.morpherPR); // gate.creole.morph.Morph
		paramsRGPR.put("caseSensitive", false);//be sure
		Gazetteer org = (DefaultGazetteer) Factory.createResource(ONTO_ROOT_GAZETTEER, paramsRGPR);
		List<String> fnames = new ArrayList<String>();
		fnames.add("Token.root");
		//-----------------------------------------------------------
		// Now we can create the Flexible Gazeteer
		paramsFlexPR = Factory.newFeatureMap();
		paramsFlexPR.put("gazetteerInst", org);
		paramsFlexPR.put("inputFeatureNames", fnames);
		
		rtn = (ProcessingResource) Factory.createResource(FLEXIBLE_GAZETTEER, paramsFlexPR);
		return rtn;
	}
	private ProcessingResource getJapeTransducer(String grammar,Ontology ontology)throws ResourceInstantiationException, MalformedURLException{
		if(ontology!=null){
			CommonMethod.screenPrint("___________________ontology URL:"+ontology.getURL());
		}
		ProcessingResource rtn = null;
		FeatureMap params = Factory.newFeatureMap();
		URL fUrl = null;
		if(mode==0){
			fUrl = new URL("file:/" + grammar);
		}else{
			fUrl = new URL(grammar);
		}
		params.put("grammarURL", fUrl);
		if(ontology != null ){
			params.put("ontology", ontology);
		}
		rtn = (ProcessingResource) Factory.createResource(JAPE_TRANSDUCER, params);
		return rtn;
	}
	private ProcessingResource getAnnieGazetteer()throws ResourceInstantiationException, MalformedURLException{
		ProcessingResource rtn = null;
		FeatureMap params = Factory.newFeatureMap();
		params = Factory.newFeatureMap();
		params.put("caseSensitive", false);
		rtn = (ProcessingResource) Factory.createResource(ANNIE_GAZETTEER, params);
		return rtn;
	}
	private ProcessingResource getHashGazetteer(String dicFile)throws ResourceInstantiationException, MalformedURLException{
		ProcessingResource rtn = null;
		FeatureMap params = Factory.newFeatureMap();
		params = Factory.newFeatureMap();
		//URL dicUrl = null;
		URL fUrl = null;
		if(mode==0){
			fUrl = new URL("file:/" + dicFile);
		}else{
			fUrl = new URL( dicFile);
		}
			params.put("caseSensitive", false);
			params.put("listsURL", fUrl);
			rtn = (ProcessingResource) Factory.createResource(HASH_GAZETTEER, params);
			return rtn;
	}
	
	public SerialAnalyserController BuildControllerPipeLine(String grammarFile)
			throws ResourceInstantiationException, MalformedURLException 
	{
		CommonMethod.screenPrint("-----------mode:"+mode+"\n  grammarFile:"+grammarFile+"\n ");
		SerialAnalyserController rtn = (SerialAnalyserController) Factory.createResource(SERIAL_CONTROLLER, Factory.newFeatureMap(),Factory.newFeatureMap(), Gate.genSym());
		//---------------load basic first----------
		ProcessingResource pr;
		pr = this.getDocumentResetPR();
		rtn.add(pr);
		pr = this.getTokenizerPR();
		rtn.add(pr);
		pr = this.getSentenceSplitterPR();
		rtn.add(pr);
		//------------load POS---------------
		pr = this.getPosPR();
		rtn.add(pr);
		pr = this.getMorphPR();
		rtn.add(pr);
		//---------------load ontology ----------
		String ontoFiles = CommonMethod.getOntoFileNames(); // D:\work\webWork\apache-tomcat-8.0.0-RC5\webapps\gateCfg\ontology\carre\carreDomainMain.owl
		
		String ontoPath , ontoJapePath;
		CommonMethod.screenPrint("mode:"+mode);
		if(mode ==0){
			ontoPath =  CommonMethod.getOntologyPathWrite();//with domain
			ontoJapePath = CommonMethod.getJapePathWrite();
		}else{
			ontoPath =  CommonMethod.getOntologyPathURL();//include domainName/
			ontoJapePath = CommonMethod.getJapePathURL();
		}
		String tmpFile,tmpFilePath,tmpOntoFile,tmpJapeFile;
		ProcessingResource tmpFlexPR,tmpJapePR;
		if(ontoFiles !=null && !ontoFiles.equals("")){
			 StringTokenizer st = new StringTokenizer(ontoFiles,",");
			 while(st.hasMoreTokens()){//for every onto file
				 tmpFile = st.nextToken();
				 tmpOntoFile = ontoPath + tmpFile+".owl";
				 CommonMethod.screenPrint("load onto file with mode:"+mode+",tmpOntoFile:"+tmpOntoFile);
				 tmpFlexPR = getFlexibleGazetteer(tmpOntoFile);
				 rtn.add(tmpFlexPR);
				 tmpJapeFile = ontoJapePath + tmpFile+".jape";//D:\work\webWork\apache-tomcat-8.0.0-RC5\webapps\gateCfg\jape\carre\carreDomainMain.jape
				 CommonMethod.screenPrint("load onto jape file:"+tmpFile+",\n ");
				 tmpJapePR = this.getJapeTransducer(tmpJapeFile, this.ontology);
				 rtn.add(tmpJapePR);
			 }
		}

		//-----------load jape--------------
		pr = this.getJapeTransducer(grammarFile, null);
		CommonMethod.screenPrint("load jape main file:"+grammarFile);
		rtn.add(pr);
		//--------------finished-------------------
		return rtn;
	}
	
	/** construct the Controller Pipeline 
	 * @throws MalformedURLException */
	public SerialAnalyserController BuildControllerPipeline0(String ontologyFile, String grammarFile,String dicFile,int mode)
			throws ResourceInstantiationException, MalformedURLException {
		//System.out.println("... Building GATE Controller Pipeline");
		// create a serial analyzer controller to run GATE with
		CommonMethod.screenPrint("-----------mode:"+mode+"\n ontologyFile:"+ontologyFile+"\n grammarFile:"+grammarFile+"\n dicFile:"+dicFile);
		SerialAnalyserController sac = (SerialAnalyserController) Factory.createResource(SERIAL_CONTROLLER, Factory.newFeatureMap(),Factory.newFeatureMap(), Gate.genSym());
		ProcessingResource prList[] = new ProcessingResource[PR_CLASSES.length];
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		// First we need to create the Ontology Language Resource
		//URL url = null;
		URL fUrl = null;
		try {
			if(mode ==0){
				fUrl = new URL("file:/" + ontologyFile);
			}else{
				fUrl = new URL( ontologyFile);
			}
			
		} catch (MalformedURLException e) {
			System.out.println(e);
			e.printStackTrace();
			//throw new GenericException(e.getMessage(), "Back", "dashboard");
		}
		params.put("rdfXmlURL", fUrl);
		Ontology ontology = (Ontology) Factory.createResource(ONTOLOGY_OWLIM,params);
		
		for (int i = 0; i < PR_CLASSES.length; i++) {
			params = Factory.newFeatureMap(); // use default parameters
			if (PR_CLASSES[i].equals(FLEXIBLE_GAZETTEER)) {
				// OK, the Flexible Gazetteer needs a lot more stuff than just
				// the
				// defaults that we can load the others with
				// Then we need to create an Onto Root Gazetteer, based on the
				// previous resources
				params = Factory.newFeatureMap();
				params.put("ontology", ontology); // gate.creole.ontology.Ontology
				params.put("tokeniser", (DefaultTokeniser) prList[2]); // gate.creole.tokeniser.DefaultTokeniser
				params.put("posTagger", (POSTagger) prList[3]); // gate.creole.POSTagger
				params.put("morpher", (Morph) prList[4]); // gate.creole.morph.Morph
				
				Gazetteer org = (DefaultGazetteer) Factory.createResource(ONTO_ROOT_GAZETTEER, params);
				List<String> fnames = new ArrayList<String>();
				fnames.add("Token.root");
				
				// Now we can create the Flexible Gazeteer
				params = Factory.newFeatureMap();
				params.put("gazetteerInst", org);
				params.put("inputFeatureNames", fnames);
			} else if (PR_CLASSES[i].equals(JAPE_TRANSDUCER)) {
				params = Factory.newFeatureMap();
				fUrl = new URL("file:/" + grammarFile);
				if(mode==0){
					fUrl = new URL("file:/" + grammarFile);
				}else{
					fUrl = new URL(grammarFile);
				}
				params.put("grammarURL", fUrl);
				//params.put("encoding", "UTF-8");
				//params.put("ontology", ontology);
			} else if (PR_CLASSES[i].equals(ANNIE_GAZETTEER)) {
				params = Factory.newFeatureMap();
				params.put("caseSensitive", false);
			} else if (PR_CLASSES[i].equals(HASH_GAZETTEER)) {
				params = Factory.newFeatureMap();
				//URL dicUrl = null;

				if(mode==0){
					fUrl = new URL("file:/" + dicFile);
				}else{
					fUrl = new URL( dicFile);
				}
					params.put("caseSensitive", false);
					params.put("listsURL", fUrl);
				
				
			}
			prList[i] = (ProcessingResource) Factory.createResource(PR_CLASSES[i], params);
		}
		for (ProcessingResource pr : prList) {
			sac.add(pr);
		}
		//System.out.println("... Pipeline complete");
		return sac;
	}

	private static String getOntologyFilePathWrite(String fileName){
		String sProject = CommonMethod.getProject();
		CommonMethod.screenPrint("project:"+sProject+",fileName:"+fileName);
		EnumProject theProject = EnumProject.valueOf(sProject);
		
		String rtn = CommonMethod.getOntologyPathWrite();//include domain name
		
		 switch (theProject) {
		    case Carre: 
		    	rtn += fileName;//ComputerGraphics.owl
		      break;   
		    case Chic: // fall through to C
		    	rtn += fileName;
		    	break;
		    case Div: 
		    	rtn += fileName;
		      break;
		  }
		 rtn+= ".owl"; 
		 return rtn;
	}
}
