package clearnlp;

import gate.util.CommonMethod;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.List;
import java.util.StringTokenizer;

//import test.mate.DemoNLPDecode;

import com.clearnlp.component.AbstractComponent;
import com.clearnlp.dependency.DEPTree;
import com.clearnlp.nlp.NLPGetter;
import com.clearnlp.nlp.NLPMode;
import com.clearnlp.reader.AbstractReader;
import com.clearnlp.segmentation.AbstractSegmenter;
import com.clearnlp.tokenization.AbstractTokenizer;
import com.clearnlp.util.UTInput;
import com.clearnlp.util.UTOutput;


public class ClearnlpDecode {

	private static ClearnlpDecode instance = null;
	private final String modelType = "general-en";//args[0];	// "general-en" or "medical-en"
	private final String language = AbstractReader.LANG_EN;
	//-----
	private AbstractTokenizer tokenizer;
	private AbstractComponent components[];
	//-------------------
	
	private ClearnlpDecode(){
		//---------------------------------
		init();
		//--------------
	}
	public static ClearnlpDecode getInstance() throws IOException{
		if(instance==null){
			instance = new ClearnlpDecode();
		}
		return instance;
	}
	public AbstractTokenizer getTokenizer(){
		return this.tokenizer;
	}
	public AbstractComponent[] getCompoents(){
		return this.components;
	}
	private void init(){
		components = new AbstractComponent[5];
		tokenizer  = NLPGetter.getTokenizer(language);
		AbstractComponent tagger;
		try {
			tagger = NLPGetter.getComponent(modelType, language, NLPMode.MODE_POS);
			AbstractComponent parser     = NLPGetter.getComponent(modelType, language, NLPMode.MODE_DEP);
			AbstractComponent identifier = NLPGetter.getComponent(modelType, language, NLPMode.MODE_PRED);
			AbstractComponent classifier = NLPGetter.getComponent(modelType, language, NLPMode.MODE_ROLE);
			AbstractComponent labeler    = NLPGetter.getComponent(modelType, language, NLPMode.MODE_SRL);
			components[0] = tagger;
			components[1] = parser;
			components[2] = identifier;
			components[3] = classifier;
			components[4] = labeler;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			CommonMethod.exceptionPrint("catched from ClearnlpDecode init:"+e.getMessage());
			e.printStackTrace();
		}

	}
	
	public SentenceRelationModel process( String sentence)
	{
		SentenceRelationModel rtn = new SentenceRelationModel();
		
		AbstractTokenizer tokenizer = instance.getTokenizer();
		AbstractComponent components[] = instance.getCompoents();
		String treeStr;
		
		DEPTree tree = NLPGetter.toDEPTree(tokenizer.getTokens(sentence));
		for (AbstractComponent component : components){
			component.process(tree);
		}
		treeStr = tree.toStringSRL();
		
        //CommonMethod.screenPrint("in ClearnlpDecode treeStr:\n"+treeStr);
        StringTokenizer st = new StringTokenizer(treeStr,"\n");
        StringTokenizer st2 ;
        String str;
       
        while(st.hasMoreTokens()){
        	str = st.nextToken();
        	
        	SentenceWordModel aWordModel = new SentenceWordModel();
        	st2 = new StringTokenizer(str);
        	int i=0;
        	while(st2.hasMoreTokens()){
        		str = st2.nextToken();
        		if(i==0){
        			aWordModel.setId(str);
        		}
        		if(i==1){//raw
        			aWordModel.setRaw(str);
        		}else if(i==2){
        			aWordModel.setLemma(str);
        		}else if(i==3){//POS
        			aWordModel.setPos(str);
        		}else if(i==4){//FEATS
        			aWordModel.setFeat(str);
        		}else if(i==5){//HEAD
        			aWordModel.setPhead(str);
        		}
        		else if(i==6){//DEPREL
        			aWordModel.setDeprel(str);
        		}else if(i==7){//SHEADS
        			if(str!=null && str.equals("_")){
        				str = "";
        			}
        			aWordModel.setSheads(str);
        		}
        		
        		i++;
        	}
        	//CommonMethod.screenPrint("("+aWordModel.toString()+")");
        	rtn.getTheSentence().add(aWordModel);
        	CommonMethod.screenPrint(" in ClearnlpDecode process SentenceRelationModel:"+aWordModel.toString());
        }
        return rtn;
	}
	
	public void process( BufferedReader reader, PrintStream fout)
	{
		AbstractTokenizer tokenizer = instance.getTokenizer();
		AbstractComponent components[] = instance.getCompoents();
		
		
		
		AbstractSegmenter segmenter = NLPGetter.getSegmenter(language, tokenizer);
		DEPTree tree;
		
		for (List<String> tokens : segmenter.getSentences(reader))
		{
			tree = NLPGetter.toDEPTree(tokens);
			
			for (AbstractComponent component : components)
				component.process(tree);
			
			fout.println(tree.toStringSRL()+"\n");
		}
		
		fout.close();
	}
	public static void main(String[] args)
	{
		//String modelType  = "general-en";//args[0];	// "general-en" or "medical-en"
		String inputFile  = "D:\\work\\Gate\\trainingModel\\tmp\\Abstract.html";//args[1];
		String outputFile = "D:\\work\\Gate\\trainingModel\\tmp\\Abstract2.html";//args[2];
		//String sentence = "I'd like to meet Dr. Choi.";
		//String sentence = "In this paper, we present an algorithm for estimating the parameters of a cloth simulation from video data of real fabric. ";
		//String sentence = "More European air time is filled with low-budget game shows , variety hours , movies and talk shows , many of which are authorized knock-offs of their American counterparts .";
		//String sentence = "An eigenspace image recognition approach is implemented on this parallel graphics architecture .";
		//String sentence = "London is captial of UK.";
		String sentence = "The Zardian government said that Gagrach wanted Zardian computers so it offered to sell some of its computer to the country";
		try
		{
			//ClearnlpDecode instance = ClearnlpDecode.getInstance();
			//instance.process( sentence);
			SentenceRelationModel models = ClearnlpDecode.getInstance().process(sentence);
		}
		catch (Exception e) {e.printStackTrace();}
	}
}
