package gate;
import java.util.*;
import java.io.*;
import java.net.*;

import ansmart.PipelineBuilder;
import gate.*;
import gate.creole.*;
import gate.util.*;
import gate.util.persistence.PersistenceManager;
import gate.corpora.RepositioningInfo;



public class GateParser {
	

	private CorpusController annieController;
	//@Value("${ontologyFile}")
	private String ontologyFile;
	//@Value("${grammarFile}")
	private String grammarFile;
	//@Value("${pluginsHome}")
	private String pluginsHome;
	//@Value("${htmlDir}")
	private String htmlDir;
	//@Value("${tohtmlUrl}")
	private String tohtmlUrl;
	//@Value("${highlighted}")
	private String highlighted;
	//@Value("${gateHome}")
	private String gateHome;
	//@Value("${uploadDir}")
	private String uploadDir;
	//@Value("${googleCustomSearchKey}")
	private String googleSearchKey;
	
	public GateParser(){
		ontologyFile = "D:"+File.separator+"work"+File.separator+"Gate"+File.separator+"myProject"+File.separator+"gateProject"+File.separator+"ontology"+File.separator+"ComputerGraphics.owl";
		grammarFile = "D:"+File.separator+"work"+File.separator+"Gate"+File.separator+"myProject"+File.separator+"gateProject"+File.separator+"jape"+File.separator+"main.jape";
		pluginsHome = "D:"+File.separator+"tools"+File.separator+"Gate"+File.separator+"gate-7.0-build4195-BIN"+File.separator+"plugins";
	}
	
	//http://gate.ac.uk/wiki/code-repository/src/sheffield/examples/StandAloneAnnie.java
	 /** The Corpus Pipeline application to contain ANNIE */
	private SerialAnalyserController annieSerialController;

	  /**
	   * Initialise the ANNIE system. This creates a "corpus pipeline"
	   * application that can be used to run sets of documents through
	   * the extraction system.
	   */
	  public void initAnnie() throws GateException, IOException {
	    Out.prln("Initialising ANNIE...");

	    // load the ANNIE application from the saved state in plugins/ANNIE
	    File pluginsHomeFile = Gate.getPluginsHome();
	    File anniePluginFile = new File(pluginsHomeFile, "ANNIE");
	    File annieGappFile = new File(pluginsHomeFile, "ANNIE_with_defaults.gapp");
	    annieController =
	      (CorpusController) PersistenceManager.loadObjectFromFile(annieGappFile);

	    Out.prln("...ANNIE loaded");
	  } // initAnnie()
		public void initSerialAnnie() {
			PipelineBuilder pipe = new PipelineBuilder(0);
			Out.prln("Initialising ANNIE...");
			//logger.info("mylogs: Initialising ANNIE...");
			
			try {
				annieController = pipe.BuildControllerPipeline0(ontologyFile,grammarFile,"",0);
			} catch (ResourceInstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				//throw new GenericException(e.getMessage(), "Back", "dashboard");
			}catch(MalformedURLException ex){
				ex.printStackTrace();
			}
			Out.prln("...ANNIE loaded");
			//logger.info("mylogs: ANNIE loaded");
		} // initAnnie()
	  /** Tell ANNIE's controller about the corpus you want to run on */
	  public void setCorpus(Corpus corpus) {
	    annieController.setCorpus(corpus);
	  } // setCorpus

	  /** Run ANNIE */
	  public void execute() throws GateException {
	    Out.prln("Running ANNIE...");
	    annieController.execute();
	    Out.prln("...ANNIE complete");
	  } // execute()

	  /**
	   * Run from the command-line, with a list of URLs as argument.
	   * <P><B>NOTE:</B><BR>
	   * This code will run with all the documents in memory - if you
	   * want to unload each from memory after use, add code to store
	   * the corpus in a DataStore.
	   */
	  public static void main(String args[]) throws GateException, IOException {
	    // initialise the GATE library
	    Out.prln("Initialising GATE...");
	    Gate.init();
	    Out.prln("...GATE initialised");
	    
		File gateHome = Gate.getGateHome();
		File pluginsHome = new File(gateHome, "plugins");
		Out.prln(pluginsHome.getAbsolutePath());
//-------------gate register
	    
		Gate.getCreoleRegister().registerDirectories(new File(pluginsHome, "ANNIE").toURL());
		Gate.getCreoleRegister().registerDirectories(new File(pluginsHome, "Gazetteer_Ontology_Based").toURL());
		Gate.getCreoleRegister().registerDirectories(new File(pluginsHome, "Ontology").toURL());
		Gate.getCreoleRegister().registerDirectories(new File(pluginsHome, "Tools").toURL());
	    
//------------------------	    
	    // initialise ANNIE (this may take several minutes)
	    GateParser annie = new GateParser();
	    //annie.initAnnie();
	    annie.initSerialAnnie();
	    // create a GATE corpus and add a document for each command-line
	    // argument
	    Corpus corpus = Factory.newCorpus("GateParser corpus");
	    
	    URL u = new URL("http://10.1.62.218:8080/heartFailure.txt");
	    FeatureMap params = Factory.newFeatureMap();
	    params.put("sourceUrl", u);
	    params.put("preserveOriginalContent", new Boolean(true));
	    params.put("collectRepositioningInfo", new Boolean(true));
	    Out.prln("Creating doc for " + u);
	    Document tmpDoc = (Document)Factory.createResource("gate.corpora.DocumentImpl", params);
	    corpus.add(tmpDoc);

	    // tell the pipeline about the corpus and run it
	    annie.setCorpus(corpus);
	    annie.execute();

	    // for each document, get an XML document with the
	    // person and location names added
	    Iterator docIter = corpus.iterator();
	    int count = 0;
	    String startTagPart_1 = "<span GateID=\"";
	    String startTagPart_2 = "\" title=\"";
	    String startTagPart_3 = "\" style=\"background:Red;\">";
	    String endTag = "</span>";
	    //loop files
	    while(docIter.hasNext()) {
	      Document doc = (Document) docIter.next();
	      AnnotationSet defaultAnnotSet = doc.getAnnotations();
	      Set annotTypesRequired = new HashSet();
	      annotTypesRequired.add("Person");
	      annotTypesRequired.add("Location");
	      //------------
	      annotTypesRequired.add("Lookup");
	      annotTypesRequired.add("KeySentence");
	      annotTypesRequired.add("Citation");
	      annotTypesRequired.add("BibliographicEntry");
	      annotTypesRequired.add("RefSection");
	      annotTypesRequired.add("cardiorenalTag");
	      annotTypesRequired.add("cardiorenalComorbiditiesTag");
	      annotTypesRequired.add("cardiorenalRiskFactorsTag");
	      annotTypesRequired.add("heartFailureWorseningRenalFunctionTag");	
	      
	      Set<Annotation> annos = new HashSet<Annotation>(defaultAnnotSet.get(annotTypesRequired));

	      FeatureMap features = doc.getFeatures();
	      String originalContent = (String)features.get(GateConstants.ORIGINAL_DOCUMENT_CONTENT_FEATURE_NAME);
	      RepositioningInfo info = (RepositioningInfo)features.get(GateConstants.DOCUMENT_REPOSITIONING_INFO_FEATURE_NAME);

	      ++count;
	      File file = new File("StANNIE_" + count + ".HTML");
	      Out.prln("File name: '"+file.getAbsolutePath()+"'");
	      
	      if(originalContent != null && info != null) {
	        Out.prln("OrigContent and reposInfo existing. Generate file...");

	        Iterator annosIt = annos.iterator();
	        Annotation currAnnot;
	        SortedAnnotationList sortedAnnotations = new SortedAnnotationList();

	        while(annosIt.hasNext()) {
	          currAnnot = (Annotation) annosIt.next();
	          sortedAnnotations.addSortedExclusive(currAnnot);
	        } // while

	        StringBuffer editableContent = new StringBuffer(originalContent);
	        long insertPositionEnd;
	        long insertPositionStart;
	        // insert anotation tags backward
	        Out.prln("Unsorted annotations count: "+annos.size());
	        Out.prln("Sorted annotations count: "+sortedAnnotations.size());
	        for(int i=sortedAnnotations.size()-1; i>=0; --i) {
	          currAnnot = (Annotation) sortedAnnotations.get(i);
	          insertPositionStart =
	            currAnnot.getStartNode().getOffset().longValue();
	          insertPositionStart = info.getOriginalPos(insertPositionStart);
	          insertPositionEnd = currAnnot.getEndNode().getOffset().longValue();
	          insertPositionEnd = info.getOriginalPos(insertPositionEnd, true);
	          if(insertPositionEnd != -1 && insertPositionStart != -1) {
	            editableContent.insert((int)insertPositionEnd, endTag);
	            editableContent.insert((int)insertPositionStart, startTagPart_3);
	            editableContent.insert((int)insertPositionStart,
	                                                          currAnnot.getType());
	            editableContent.insert((int)insertPositionStart, startTagPart_2);
	            editableContent.insert((int)insertPositionStart,
	                                                  currAnnot.getId().toString());
	            editableContent.insert((int)insertPositionStart, startTagPart_1);
	          } // if
	        } // for

	        FileWriter writer = new FileWriter(file);
	        writer.write(editableContent.toString());
	        writer.close();
	      } // if - should generate
	      else if (originalContent != null) {
	        Out.prln("OrigContent existing. Generate file...");

	        Iterator it = annos.iterator();
	        Annotation currAnnot;
	        SortedAnnotationList sortedAnnotations = new SortedAnnotationList();

	        while(it.hasNext()) {
	          currAnnot = (Annotation) it.next();
	          sortedAnnotations.addSortedExclusive(currAnnot);
	        } // while

	        StringBuffer editableContent = new StringBuffer(originalContent);
	        long insertPositionEnd;
	        long insertPositionStart;
	        // insert anotation tags backward
	        Out.prln("Unsorted annotations count: "+annos.size());
	        Out.prln("Sorted annotations count: "+sortedAnnotations.size());
	        for(int i=sortedAnnotations.size()-1; i>=0; --i) {
	          currAnnot = (Annotation) sortedAnnotations.get(i);
	          insertPositionStart =
	            currAnnot.getStartNode().getOffset().longValue();
	          insertPositionEnd = currAnnot.getEndNode().getOffset().longValue();
	          if(insertPositionEnd != -1 && insertPositionStart != -1) {
	            editableContent.insert((int)insertPositionEnd, endTag);
	            editableContent.insert((int)insertPositionStart, startTagPart_3);
	            editableContent.insert((int)insertPositionStart,
	                                                          currAnnot.getType());
	            editableContent.insert((int)insertPositionStart, startTagPart_2);
	            editableContent.insert((int)insertPositionStart,
	                                                  currAnnot.getId().toString());
	            editableContent.insert((int)insertPositionStart, startTagPart_1);
	          } // if
	        } // for

	        FileWriter writer = new FileWriter(file);
	        writer.write(editableContent.toString());
	        writer.close();
	      }
	      else {
	        Out.prln("Content : "+originalContent);
	        Out.prln("Repositioning: "+info);
	      }

	      String xmlDocument = doc.toXml(annos, false);
	      String fileName = new String("StANNIE_toXML_" + count + ".HTML");
	      FileWriter writer = new FileWriter(fileName);
	      writer.write(xmlDocument);
	      writer.close();

	    } // for each doc
	  } // main

	  /**
	   *
	   */
	  public static class SortedAnnotationList extends Vector {
	    public SortedAnnotationList() {
	      super();
	    } // SortedAnnotationList

	    public boolean addSortedExclusive(Annotation annot) {
	      Annotation currAnot = null;

	      // overlapping check
	      for (int i=0; i<size(); ++i) {
	        currAnot = (Annotation) get(i);
	        if(annot.overlaps(currAnot)) {
	          return false;
	        } // if
	      } // for

	      long annotStart = annot.getStartNode().getOffset().longValue();
	      long currStart;
	      // insert
	      for (int i=0; i < size(); ++i) {
	        currAnot = (Annotation) get(i);
	        currStart = currAnot.getStartNode().getOffset().longValue();
	        if(annotStart < currStart) {
	          insertElementAt(annot, i);
	          /*
	           Out.prln("Insert start: "+annotStart+" at position: "+i+" size="+size());
	           Out.prln("Current start: "+currStart);
	           */
	          return true;
	        } // if
	      } // for

	      int size = size();
	      insertElementAt(annot, size);
	//Out.prln("Insert start: "+annotStart+" at size position: "+size);
	      return true;
	    } // addSorted
	  } // SortedAnnotationList
	} // class StandAloneAnnie
	 
	
	

