/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package javalda;

import java.io.*;
import java.util.*;

/**
 *
 * @author Alfan Farizki Wicaksono
 * Institut Teknologi Bandung
 * Bandung, Indonesia.
 * 
 * Toy Implementation of LDA using Gibbs Sampling
 */

/*
 * Corpus format assumption:
 * =========================
 * suppose we have two data:
 * 1. collection (a number of documents)
 * 2. vocabulary list (distinct word)
 * 
 * collection           
 * -------------  
 * ###
 * word1    doc1
 * word2    doc1
 * ###
 * word3    doc2
 * ..       ..
 * ###
 * wordn    docn
 * 
 * vocabulary
 * -------------
 * word1
 * word2
 * ..
 * wordn
 */
public class Corpus {
    
    /*mapping between word and index*/
    List<String> vocabulary = new ArrayList<String>();
    
    /*list of documents*/
    List<document> docs = new ArrayList<document>();
    
    public Corpus(String vocabFile, String collectionFile) {
        this.loadVocabulary(vocabFile);
        this.loadDocuments(collectionFile);
    }
    
    /*a document*/
    public class document {
        
        /*key:word (vocab index), value:topic*/
        public Map<Integer, Integer> word_topic = new HashMap<Integer, Integer>();
    }
   
    public void loadVocabulary(String vocabFile) {
        try {
            FileInputStream fstream = new FileInputStream(vocabFile);
            DataInputStream in = new DataInputStream(fstream);
            BufferedReader br = new BufferedReader(new InputStreamReader(in));
            String strLine;

            //add the vocabulary word by word
            while ((strLine = br.readLine()) != null)   {
                this.vocabulary.add(strLine);
            }

            in.close();
        } catch (Exception e){
            System.err.println("Error: " + e.getMessage());
        }
    }
    
    //must be run after loadVocabulary
    public void loadDocuments(String collectionFile) {
        try {
            FileInputStream fstream = new FileInputStream(collectionFile);
            DataInputStream in = new DataInputStream(fstream);
            BufferedReader br = new BufferedReader(new InputStreamReader(in));
            String strLine;

            //add the vocabulary word by word
            strLine = br.readLine();
            while (strLine != null)   {
                while(strLine != null && strLine.equals("###")) {
                    strLine = br.readLine();
                }
                while(strLine != null && !strLine.equals("###")) {
                    int index = this.vocabulary.lastIndexOf(strLine);
                    document d = new document();
                    d.word_topic.put(index, 0);
                    this.docs.add(d);
                    
                    strLine = br.readLine();
                }
            }

            in.close();
        } catch (Exception e){
            System.err.println("Error: " + e.getMessage());
        }
    }
    
    /*get the number of documents in the collection*/
    public int getNumDocs() {
        return this.docs.size();
    }
    
    /*get the size of vocabulary*/
    public int getNumVocab() {
        return this.vocabulary.size();
    }
}
