/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package javalda;

import java.util.*;
import java.util.Map.*;

/**
 *
 * @author Alfan Farizki Wicaksono
 * Institut Teknologi Bandung
 * Bandung, Indonesia.
 * 
 * Toy Implementation of LDA using Gibbs Sampling
 */

public class Util {
    
    /*generate sample: random sample based on probability p_z*/
    /*int:topic index, double:topic probability*/
    public static int generateSample(Map<Integer, Double> p_z) {
        Random generator = new Random();
        double r = generator.nextDouble();
        int sample = 0;
        
        double cumulative = 0.0d;
        
        for (Entry e: p_z.entrySet()) {
            cumulative += (Double)e.getValue();
            if (r < cumulative)
            {
                sample = (Integer)e.getKey();
                break;
            }
        }
        
        return sample;
    }
    
    /*random: uniformly distributed*/
    public static int generateTopicRandom(int num_topic) {
        Random gen = new Random();
        int rand_z = gen.nextInt(num_topic);
        
        return rand_z % num_topic;
    }
    
    /*LOG GAMMA FUNCTION*/
    public static double logGamma(double x) {
      double tmp = (x - 0.5) * Math.log(x + 4.5) - (x + 4.5);
      double ser = 1.0 + 76.18009173    / (x + 0)   - 86.50532033    / (x + 1)
                       + 24.01409822    / (x + 2)   -  1.231739516   / (x + 3)
                       +  0.00120858003 / (x + 4)   -  0.00000536382 / (x + 5);
      return tmp + Math.log(ser * Math.sqrt(2 * Math.PI));
    }
   
    /*LOG DIRICHLET DELTA FUNCTION - input as a vector*/
    public static double logDeltaVector(double[] x) {
        
        double delta     = 0.0d;
        double vectorSum = 0.0d;
        
        for (int i = 0; i < x.length; i++) {
            delta += logGamma(x[i]);
            vectorSum += x[i];
        }
        
        delta -= logGamma(vectorSum);
        
        return delta;
    }
    
    /*LOG DIRICHLET DELTA FUNCTION - input as a scalar*/
    public static double logDeltaScalar(int dim, double x) {
        
        double delta = dim*logGamma(x) - logGamma((double)dim * x);

        return delta;
    }
}
