package infoMining;

import java.util.List;
import java.util.Map;
import java.util.Properties;

import edu.stanford.nlp.dcoref.CorefChain;
import edu.stanford.nlp.dcoref.CorefCoreAnnotations.CorefChainAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.NamedEntityTagAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.PartOfSpeechAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.SentencesAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.TextAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.TokensAnnotation;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.semgraph.SemanticGraph;
import edu.stanford.nlp.semgraph.SemanticGraphCoreAnnotations.CollapsedCCProcessedDependenciesAnnotation;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.TreeCoreAnnotations.TreeAnnotation;
import edu.stanford.nlp.util.CoreMap;

public class TestCoreference {
	public static void main(String args[]){
		// creates a StanfordCoreNLP object, with POS tagging, lemmatization, NER, parsing, and coreference resolution 
	    Properties props = new Properties();
	    props.setProperty("annotators", "tokenize, ssplit, pos, lemma, ner, parse, dcoref");
	    StanfordCoreNLP pipeline = new StanfordCoreNLP(props);
	   
	    // read some text in the text variable
	   String text = "We report here 24 new Wilms tumor (WT) patients with germline WT1 alterations and a synopsis of our own previously described and literature cases in whom age of tumor-onset, gender, and laterality were known. This combined database contains 282 patients, 117 patients with and 165 without WT1 germline alterations. Using this information we have determined the median age of tumor-onset for patients with (12.5 months) and without WT1 gene alterations (36 months). The earliest onset was in patients with truncation (12 mo, 66 patients), followed by missense mutations (18 mo, 30 patients) and deletions (22 mo, 21 patients). Patients with the two most frequent nonsense mutations R362X and R390X and the Denys-Drash syndrome (DDS) hot spot mutation R394W/Q/L had a very early onset (9, 12, and 18 mo, respectively). The highest number of bilateral tumors was observed in the group of truncation mutations, with a higher percentage of bilateral tumors when truncations occurred in the 5' half of the WT1 gene. In addition to genital tract anomalies (GU), early onset nephrotic syndrome with diffuse mesangial sclerosis and stromal-predominant histology, tumor bilaterality, and early age of onset can now be added to the list of risk factors for carrying a germline WT1 mutation. ";
	   System.out.println("paper title:"+"Twenty-four new cases of WT1 germline mutations and review of the literature: genotype/phenotype correlations for Wilms tumor development."); 
	   Annotation document = new Annotation(text);//create an empty annotation
	    
	    // run all Annotators on this text
	    pipeline.annotate(document);
	    
	    // these are all the sentences in this document
	    // a CoreMap is essentially a Map that uses class objects as keys and has values with custom types
	    List<CoreMap> sentences = document.get(SentencesAnnotation.class);
	    
	    for(CoreMap sentence: sentences) {
	      // traversing the words in the current sentence
	      // a CoreLabel is a CoreMap with additional token-specific methods
	    	System.out.println("---------------sentence:"+sentence.toString());
	      for (CoreLabel token: sentence.get(TokensAnnotation.class)) {
	        // this is the text of the token
	        String word = token.get(TextAnnotation.class);
	        // this is the POS tag of the token
	        String pos = token.get(PartOfSpeechAnnotation.class);
	        // this is the NER label of the token
	        String ne = token.get(NamedEntityTagAnnotation.class);       
	        System.out.println("word:"+word+",pos:"+pos+",ne:"+ne);
	      }

	      // this is the parse tree of the current sentence
	      Tree tree = sentence.get(TreeAnnotation.class);

	      // this is the Stanford dependency graph of the current sentence
	      SemanticGraph dependencies = sentence.get(CollapsedCCProcessedDependenciesAnnotation.class);
	      System.out.print(dependencies.toString());
	    }

	    // This is the coreference link graph
	    // Each chain stores a set of mentions that link to each other,
	    // along with a method for getting the most representative mention
	    // Both sentence and token offsets start at 1!
	    	Map<Integer, CorefChain> graph = document.get(CorefChainAnnotation.class);
	    /**/
	}
}
